<?php
class aiosc_Settings {
    public $settings;
    public $defaults;
    public $meta_key;
    function __construct() {
        $this->meta_key = 'aiosc_settings';
        $this->defaults = $this->get_defaults();
        $this->load_settings();
    }
    function load_settings() {
        $this->settings = get_option($this->meta_key);
    }
    function save_settings() {
        update_option($this->meta_key,$this->settings);
    }
    function get($key,$default=true) {
        $key = strtolower($key);
        if(isset($this->settings) && isset($this->settings[$key])) {
            if($key == 'min_content_len' || $key == 'min_reply_len') {
                if($this->settings[$key] < $this->defaults[$key]) return $this->defaults[$key];
                else return $this->settings[$key];
            }
            return @$this->settings[$key];
        }
        else {
            if($default) {
                if(isset($this->defaults[$key])) return $this->defaults[$key];
                else return '';
            }
            return '';
        }
    }
    function set($key,$value, $save=false) {
        $key = strtolower($key);
        $this->settings[$key] = $value;
        if($save) {
            $this->save_settings();
        }
    }
    /**
     * @return array
     */
    private function get_defaults() {
        $settings = array(
            //ticket settings
            'min_subject_len'=>10, //0 == not required. Minimum string length for Subject
            'min_content_len'=>10, //always required. if 0, minimum will be 20.
            'min_reply_len'=>10, //always required. if 0, minimum will be 10
            'allow_upload'=>true, //allow users to upload files to their tickets & replies
            'allow_download'=>true, //allow users to download files from tickets & replies
            'upload_mimes'=>'txt,doc,docx,rtf,pdf,jpg,png,bmp,gif,rtf,zip,rar,gz,7z,mp3,sql', //* == all are allowed, but PHP, BAT and EXE will always be disallowed for security reasons.
            'upload_mimes_forbid'=>false, //if true, upload_mimes will be FORBIDDEN, instead of ONLY ALLOWED
            'max_upload_size_per_file'=>4 * 1024, /** in kilobytes (but @max_upload_size must be set in php.ini as well) */
            'max_files_per_ticket'=>4, //minimum: 1 - how many files can user attach on a single ticket?
            'max_files_per_reply'=>4, //minimum: 1 - how many files can user attach on a single reply?
            'creation_delay'=>60, //in seconds. How much time needs to pass before user can create new ticket
            'reply_delay'=>20, //in seconds. How much time needs to pass before user can post new reply (does not apply to staff members)
            "max_queue_exclusive"=>10,//how many Queued tickets can Exclusive Customer have at the time? 0 == infinite
            "max_queue_customer"=>5, //how many Queued tickets can Customer have at the time? 0 == infinite
            "max_open_exclusive"=>10, //how many Open tickets can Exclusive Customer have at the time? 0 == infinite
            "max_open_customer"=>5, //how many Open tickets can Customer have at the time? 0 == infinite
            //role settings
            'default_role'=>'aiosc_customer', //user role that will be set to user on every new user registration
            //general
            'enable_hints'=>false, //show tooltips next to input fields
            'enable_staff_ribbon'=>true,
            'enable_public_tickets'=>false,

            //pages
            'pages_frontend_enable'=>false,
            'page_ticket_form'=>0,
            'page_ticket_preview'=>0,
            'page_ticket_list'=>0,
            //email
            'email_piping_enable'=>false,
            'email_piping_domain'=>'',
            'email_piping_enable_html'=>true,
            //email responders
            'email_ar_customer_ticket_creation'=>true,
            'email_ar_customer_ticket_reply'=>true,
            'email_ar_customer_ticket_close'=>true,
            'email_ar_staff_ticket_creation'=>true,
            'email_ar_staff_ticket_reply'=>true,
            'email_ar_staff_ticket_close'=>true,
            //email templates
            'email_templates_customer_creation_subject'=>aiosc_get_data_file('emails/et_customer_creation_subject.html'),
            'email_templates_customer_creation_content'=>aiosc_get_data_file('emails/et_customer_creation_content.html'),
            'email_templates_customer_reply_subject'=>aiosc_get_data_file('emails/et_customer_reply_subject.html'),
            'email_templates_customer_reply_content'=>aiosc_get_data_file('emails/et_customer_reply_content.html'),
            'email_templates_customer_closure_subject'=>aiosc_get_data_file('emails/et_customer_closure_subject.html'),
            'email_templates_customer_closure_content'=>aiosc_get_data_file('emails/et_customer_closure_content.html'),

            'email_templates_staff_creation_subject'=>aiosc_get_data_file('emails/et_staff_creation_subject.html'),
            'email_templates_staff_creation_content'=>aiosc_get_data_file('emails/et_staff_creation_content.html'),
            'email_templates_staff_reply_subject'=>aiosc_get_data_file('emails/et_staff_reply_subject.html'),
            'email_templates_staff_reply_content'=>aiosc_get_data_file('emails/et_staff_reply_content.html'),
            'email_templates_staff_closure_subject'=>aiosc_get_data_file('emails/et_staff_closure_subject.html'),
            'email_templates_staff_closure_content'=>aiosc_get_data_file('emails/et_staff_closure_content.html'),
        );
        $def = apply_filters('aiosc_defaults',$settings);
        if(empty($def)) $def = $settings;
        return $def;
    }
    /**
     * @called-on-activation
     */
    function install() {
        if(get_option($this->meta_key,false) == false) update_option($this->meta_key,$this->defaults);
        $this->load_settings();
        aiosc_log('[AIOSC Installer] [Settings] Installed successfully.');
    }

    /**
     * @called-on-uninstall
     */
    function uninstall() {
        delete_option($this->meta_key);
    }

}

global $aiosc_settings;
$aiosc_settings = new aiosc_Settings();

function aiosc_get_settings($key, $default='') {
    global $aiosc_settings;
    return $aiosc_settings->get($key, $default);
}
function aiosc_save_settings() {
    global $aiosc_settings;
    $aiosc_settings->save_settings();
}
function aiosc_set_settings($key, $value, $save=true) {
    global $aiosc_settings;
    $aiosc_settings->set($key, $value, $save);
}