<?php
/**
 * Last update: 1.0.6
 */
class aiosc_Shortcodes {
    /**
     * Create New Ticket form shortcode
     * @update 1.0.6
     * @update 1.1 - added @aiosc_enqueue_front hook
     * @param $atts
     * @return string
     */
    static function ticket_form($atts) {
        global $aiosc_user, $aiosc_settings;
        wp_enqueue_style('aiosc-frontend',AIOSC_URL.'assets/css/front.css');
        if($aiosc_user->can('create_ticket')) {
            global $priorities, $departments;
            $departments = aiosc_DepartmentManager::get_departments(true);
            $priorities = aiosc_PriorityManager::get_priorities(true);

            wp_enqueue_script('jquery');
            wp_enqueue_script('jquery-form');
            wp_enqueue_script('jquery-browser',AIOSC_URL.'assets/js/jquery.browser.min.js');
            wp_enqueue_script('aiosc-common',AIOSC_URL.'assets/js/common.js');
            wp_enqueue_script('aiosc-uploader',AIOSC_URL.'assets/js/uploader.js');
            wp_enqueue_script('aiosc-new-ticket',AIOSC_URL.'assets/js/new-ticket.js');
            do_action('aiosc_enqueue_front');

            return aiosc_load_template('shortcodes/ticket_form.php',true);

        }
        else {
            return aiosc_load_template('shortcodes/ticket_form_error.php', true);
        }
    }

    /**
     * @shortcode [aiosc_ticket_list publicOnly=false]
     * if @publicOnly is true, table will display only tickets with Public visibility and will
     * ignore user's tickets, which means this should be used on a "FAQ" page for example.
     * Otherwise, table will display user's tickets (where user is author)
     * @update 1.0.6
     * @update 1.1 - added @aiosc_enqueue_front hook
     * @param $atts
     * @return string
     */
    static function ticket_list($atts) {
        extract( shortcode_atts( array(
            "publiconly"=>false
        ), $atts ) );
        global $aiosc_user, $aiosc_settings;
        wp_enqueue_style('aiosc-frontend',AIOSC_URL.'assets/css/front.css');
        if($aiosc_user->can('create_ticket')) {
            global $tickets,$public_mode;
            $public_mode = ("{$publiconly}" == true)?true:false;

            wp_enqueue_script('jquery');
            wp_enqueue_script('jquery-form');
            wp_enqueue_script('jquery-browser',AIOSC_URL.'assets/js/jquery.browser.min.js');
            wp_enqueue_style('dashicons');
            wp_enqueue_script('aiosc-common',AIOSC_URL.'assets/js/common.js');
            wp_enqueue_script('aiosc-list',AIOSC_URL.'assets/js/tickets-list.js');
            do_action('aiosc_enqueue_front');
            return aiosc_load_template('shortcodes/ticket_list.php',true);
        }
        else {
            return aiosc_load_template('shortcodes/ticket_list_error.php',true);
        }
    }
    /**
     * Singular ticket preview shortcode
     * @update 1.0.6
     * @update 1.1 - added @aiosc_enqueue_front hook
     * @param $atts
     * @return string
     */
    static function ticket_preview($atts) {
        extract( shortcode_atts( array(
            "ticket_id"=>0
        ), $atts ) );
        global $aiosc_user, $aiosc_settings, $ticket;
        $ticket = isset($_GET['ticket_id']) && is_numeric($_GET['ticket_id'])?$_GET['ticket_id']:"{$ticket_id}";
        $ticket = new aiosc_Ticket($ticket);
        wp_enqueue_style('aiosc-frontend',AIOSC_URL.'assets/css/front.css');
        if($aiosc_user->can('view_ticket',array('ticket_id'=>$ticket))) {
            global $replies, $department, $priority, $author, $operator;
            $reply_limit = apply_filters('aiosc_reply_limit',3);
            $replies = aiosc_ReplyManager::get_replies(array('ticket_id'=>$ticket->ID),' ORDER BY ID DESC LIMIT 0,'.$reply_limit);
            $department = new aiosc_Department($ticket->department_id);
            $priority = new aiosc_Priority($ticket->priority_id);
            $author = new aiosc_User($ticket->author_id);
            $operator = new aiosc_User($ticket->op_id);
            wp_enqueue_script('jquery');
            wp_enqueue_script('jquery-form');
            wp_enqueue_script('jquery-browser',AIOSC_URL.'assets/js/jquery.browser.min.js');
            wp_enqueue_script('aiosc-common',AIOSC_URL.'assets/js/common.js');
            wp_enqueue_script('aiosc-uploader',AIOSC_URL.'assets/js/uploader.js');
            wp_enqueue_script('aiosc-downloader',AIOSC_URL.'assets/js/downloader.js');
            wp_enqueue_script('aiosc-ticket-preview',AIOSC_URL.'assets/js/ticket-preview.js');
            do_action('aiosc_enqueue_front');

            return aiosc_load_template('shortcodes/ticket_preview.php',true);

        }
        else {
            return aiosc_load_template('shortcodes/ticket_preview_error.php',true);
        }
    }
}