<?php
/*
 * Plugin Name: AIO Support Center
 * Plugin Class Name: aiosc
 * Description: All-in-One solution for providing awesome support to your clients.
 * Author: 20script
 * Version: 1.0.9
 * Plugin URI: http://diwave-coders.com/plugins/aio-support-center-wordpress-ticketing-system/
 * Support URI: http://diwave-coders.com/support?department=2
 * Author URI: http://www.20script.ir/
 * Text Domain: aiosc
*/

//Defines
define('AIOSC_ERROR_LOGGING', true);
define('AIOSC_DEBUG',true); //for javascript debugging
define('AIOSC_DEMO_MODE',false); //for preview
define('AIOSC_DIR',plugin_dir_path(__FILE__));
define('AIOSC_URL',plugin_dir_url(__FILE__));
define('AIOSC_UPLOAD_DIR',apply_filters('aiosc_upload_dir',AIOSC_DIR."uploads"));
define('AIOSC_UPLOAD_URL',apply_filters('aiosc_upload_url',AIOSC_URL."uploads"));
define('AIOSC_PERMISSION_ERROR',__("<strong>Permission Error:</strong> You do not have permission for this action.",'aiosc'));

/** Textdomain Registration */
function aiosc_load_plugin_textdomain() {
    $locale = apply_filters( 'plugin_locale', get_locale(), 'aiosc' );
    load_plugin_textdomain( 'aiosc', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );
}
add_action( 'init', 'aiosc_load_plugin_textdomain' );

//Init Variables
require_once AIOSC_DIR."/inc/vars.php";

//Init Tools
require_once AIOSC_DIR."/inc/tools.php";

//Classes
require_once AIOSC_DIR.'/class/class.capabilities.php';
require_once AIOSC_DIR.'/class/class.settings.php';

require_once AIOSC_DIR.'/class/class.user.php';
require_once AIOSC_DIR.'/class/class.user-manager.php';

require_once AIOSC_DIR.'/class/class.department.php';
require_once AIOSC_DIR.'/class/class.department-manager.php';

require_once AIOSC_DIR.'/class/class.priority.php';
require_once AIOSC_DIR.'/class/class.priority-manager.php';

require_once AIOSC_DIR.'/class/class.attachment.php';
require_once AIOSC_DIR.'/class/class.attachment-manager.php';

require_once AIOSC_DIR.'/class/class.premade-response.php';
require_once AIOSC_DIR.'/class/class.premade-response-manager.php';

require_once AIOSC_DIR.'/class/class.ticket.php';
require_once AIOSC_DIR.'/class/class.ticket-manager.php';

require_once AIOSC_DIR.'/class/class.reply.php';
require_once AIOSC_DIR.'/class/class.reply-manager.php';

require_once AIOSC_DIR.'/class/class.shortcodes.php';

//require_once AIOSC_DIR.'/class/class.email.php';
require_once AIOSC_DIR.'/class/class.email-manager.php';

require_once AIOSC_DIR.'/class/class.addon-manager.php';

//Init Hooks & Pages
require_once AIOSC_DIR.'/inc/hooks.php';
require_once AIOSC_DIR.'/inc/hooks_tools.php';
require_once AIOSC_DIR.'/inc/pages.php';

//more-addons tab
require_once AIOSC_DIR.'/inc/more-addons.php';

/**
 * Main Plugin Class
 * Class aiosc
 */
/**
 * Installation hook, had to move it outside of class
 * because class is called on WP init, however registration
 * hooks are executed before WP init.
 * @since 1.0.81
 */
register_activation_hook(__FILE__,'aiosc_install');
function aiosc_install() {
    global $aiosc_capabilities, $aiosc_settings;
    $aiosc_capabilities->install();
    $aiosc_settings->install();

    //Install DB tables
    $file = aiosc_get_data_file('sql/install.php');
    $queries = explode(';',$file);

    global $wpdb;
    $y = $cnt = 0;
    $wpdb->hide_errors();
    foreach($queries as $query) {
        if(strlen(trim($query)) > 2) {
            $query = str_replace("{%wp_prefix%}",$wpdb->prefix,$query);
            try {
                if($wpdb->query($query))
                    $y++;
            }
            catch(Exception $e) {}
            $cnt++;
        };
    }
    $wpdb->show_errors();
    aiosc_log("[AIOSC Installer] Total of $y queries were executed, out of $cnt.");

    //aiosc_UserManager::install_meta();
    aiosc_DepartmentManager::install();
    aiosc_PriorityManager::install();
}
class aiosc {
    function __construct() {
        new aiosc_Hooks();
        new aiosc_HooksTools();
        new aiosc_Pages();

        do_action('aiosc_init');
    }
}

/**
 * Initialize AIOSC on WP Init to make sure every component required by AIOSC is loaded
 * before AIOSC initializes.
 * @since 1.0.8
 */
function aiosc_initialize() {
    new aiosc();
}
add_action('init','aiosc_initialize');
